package cn.palmte.work.controller.backend;

import cn.palmte.work.bean.ApproveStatusEnum;
import cn.palmte.work.bean.BudgetBean;
import cn.palmte.work.bean.EstimateBean;
import cn.palmte.work.bean.ResponseMsg;
import cn.palmte.work.model.*;
import cn.palmte.work.service.ProjectBudgetService;
import cn.palmte.work.service.ProjectEstimateService;
import cn.palmte.work.service.ProjectService;
import cn.palmte.work.utils.FreeMarkerUtil;
import cn.palmte.work.utils.InterfaceUtil;
import cn.palmte.work.utils.Utils;
import cn.palmte.work.utils.excel.ExportUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.format.datetime.DateFormatter;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import top.jfunc.common.db.bean.Page;
import top.jfunc.common.utils.CollectionUtil;

import javax.persistence.criteria.CriteriaBuilder;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 项目管理功能
 * @author xiongshiyan at 2021/10/29 , contact me with email yanshixiong@126.com or phone 15208384257
 */
@Controller
@RequestMapping("/project")
public class ProjectController extends BaseController{
    private static final Logger logger = LoggerFactory.getLogger(ProjectController.class);

    @Autowired
    private DeptRepository deptRepository;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private ProjectEstimateService projectEstimateService;
    @Autowired
    private ProjectBudgetService projectBudgetService;

    /**
     * 项目列表
     */
    @RequestMapping("/list")
    public String list(@RequestParam(value = "keywords",required = false) String keywords,
                       @RequestParam(value = PAGE_NUMBER, defaultValue = DEFAULT_PAGE_NUMBER) int pageNumber,
                       @RequestParam(value = PAGE_SIZE, defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                       Map<String, Object> model) {
        //当前登录人的角色类型
        model.put("keywords",keywords);
        model.put("deptList", deptRepository.findAll());
        ConcurrentHashMap<String, String> searchInfo = getSearchInfo(keywords,model);
        model.put("pager",projectService.list(searchInfo,pageNumber,pageSize));
        /*Page<Project> myApproveProjects = projectService.findMyApproveProjects(searchInfo, InterfaceUtil.getAdminId(), 1, Integer.MAX_VALUE);
        model.put("hasApproveProjects", CollectionUtil.isNotEmpty(myApproveProjects.getList()) ? "1" : "0");*/
        return "admin/project_list";
    }

    /**
     * 项目导出
     */
    @RequestMapping("/export")
    public void export(@RequestParam(value = "keywords",required = false) String keywords, HttpServletResponse httpServletResponse) throws IOException {
        Map<String, String> searchInfo = getSearchInfo(keywords);
        downloadHeader(httpServletResponse , Utils.generateExcelName("项目报表"), "application/octet-stream");
        String[] headers = {"项目名称","项目类型","项目状态","审核状态","当前审核人","项目创建者","部门名称","项目开始时间","项目结束时间","最后更新时间"};
        String[] exportColumns = {"name","typeDesc","statusDesc","approveStatusDesc","approveName","creatorName","deptName","startDate","endDate","lastUpdateTime"};
        ExportUtils.exportToExcel(headers, exportColumns, 1, 10000,
                httpServletResponse.getOutputStream(), (pN, pS) -> projectService.list(searchInfo, pN, pS).getList());
    }


    /**
     * 新增，至项目概算
     */
    @RequestMapping("/add")
    public String add(Map<String, Object> model) {

        //TODO 根据登录人的部门
        Dept one = deptRepository.findOne(1);
        model.put("dept",one);

        return "admin/project_estimate_add";
    }

    /**
     * 预算保存草稿
     */
    @RequestMapping("/estimateAddSave")
    public String estimateAddSave(Project project, EstimateBean estimateBean, Map<String, Object> model) {
        projectService.estimateAddSave(project, estimateBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_UNCOMMIT);
        return "redirect:/project/list";
    }

    /**
     * 预算保存并提交审核
     */
    @RequestMapping("/estimateAddSaveAndApprove")
    public String estimateAddSaveAndApprove(Project project, EstimateBean estimateBean, Map<String, Object> model) {
        projectService.estimateAddSave(project, estimateBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_PENDING);
        //TODO 发起审核
        return "redirect:/project/list";
    }


    @RequestMapping("/edit")
    public String edit(@RequestParam("id") int id,Map<String, Object> model) {
        Project project = projectService.getProject(id);
        model.put("project", project);
        EstimateBean estimateBean = projectEstimateService.getEstimate(project);
        model.put("estimateBean", estimateBean);
        //freemarker可以利用的静态方法
        model.put("Utils", FreeMarkerUtil.fromStaticPackage("cn.palmte.work.utils.Utils"));
        return "admin/project_estimate_edit";
    }
    @RequestMapping("/estimateEditSave")
    public String estimateEditSave(Project project, EstimateBean estimateBean, Map<String, Object> model) {
        projectService.estimateEditSave(project, estimateBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_UNCOMMIT);
        return "redirect:/project/list";
    }

    @RequestMapping("/estimateEditSaveAndApprove")
    public String estimateEditSaveAndApprove(Project project, EstimateBean estimateBean, Map<String, Object> model) {
        projectService.estimateEditSave(project, estimateBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_PENDING);
        //TODO 发起审核
        return "redirect:/project/list";
    }

    /**
     * 进项目预算页面
     */
    @RequestMapping("/budgetEdit")
    public String budget(@RequestParam("id") int id,Map<String, Object> model) {
        Project project = projectService.getProject(id);
        model.put("project", project);
        BudgetBean budgetBean = projectBudgetService.getBudget(project);
        //预算主页面数据
        model.put("budgetBean", budgetBean);
        //收入明细
        model.put("incomeDetails", projectBudgetService.getBudgetIncomeDetail(project));
        //成本明细
        model.put("costDetails", projectBudgetService.getBudgetCostDetail(project));
        //项目管理成本明细
        model.put("costProjectManageDetails", projectBudgetService.getBudgetCostProjectManageDetail(project));
        List<ProjectBudgetPlanDetail> projectBudgetPlanDetails = projectBudgetService.getProjectBudgetPlanDetails(project);
        //资金计划明细
        model.put("projectBudgetPlanDetails", projectBudgetPlanDetails);
        //资金计划总【上面汇总表】
        model.put("projectBudgetPlanDetailTotal", projectBudgetService.getProjectBudgetPlanDetailTotal(project, projectBudgetPlanDetails));
        //资金计划表中的统计信息【下面资金小表】
        model.put("underwrittenPlanStatistic", projectBudgetService.getProjectUnderwrittenPlanStatisticBean(projectBudgetPlanDetails));
        //现金表
        model.put("cashFlowBean", projectBudgetService.getCashFlowBean(project, projectBudgetPlanDetails));
        //freemarker可以利用的静态方法
        model.put("Utils", FreeMarkerUtil.fromStaticPackage("cn.palmte.work.utils.Utils"));
        return "admin/project_budget_edit";
    }

    /**
     * 进行项目预算保存
     */
    @RequestMapping("/budgetEditSave")
    public String budgetEditSave(Project project, BudgetBean budgetBean, Map<String, Object> model) {
        projectService.budgetEditSave(project, budgetBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_UNCOMMIT);
        return "redirect:/project/list";
    }
    /**
     * 进行项目预算保存并提交审核
     */
    @RequestMapping("/budgetEditSaveAndApprove")
    public String budgetEditSaveAndApprove(Project project, BudgetBean budgetBean, Map<String, Object> model) {
        projectService.budgetEditSave(project, budgetBean, InterfaceUtil.getAdmin(), ApproveStatusEnum.APPROVAL_PENDING);
        return "redirect:/project/list";
    }

    /**
     * 保存收入明细
     */
    @RequestMapping("/budgetEditSaveIncomeDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveIncomeDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetIncomeDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetIncomeDetail detail = details.getObject(i, ProjectBudgetIncomeDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetIncomeDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }
    /**
     * 保存成本明细
     */
    @RequestMapping("/budgetEditSaveCostDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveCostDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetCostDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetCostDetail detail = details.getObject(i, ProjectBudgetCostDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetCostDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }
    /**
     * 保存项目管理成本明细
     */
    @RequestMapping("/budgetEditSaveCostProjectManageDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveCostProjectManageDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetCostProjectManageDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetCostProjectManageDetail detail = details.getObject(i, ProjectBudgetCostProjectManageDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetCostProjectManageDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }
    /**
     * 保存项资金计划明细
     */
    @RequestMapping("/budgetEditSaveBudgetPlanDetail")
    @ResponseBody
    public ResponseMsg budgetEditSaveBudgetPlanDetail(@RequestBody String body) {
        JSONObject jsonObject = JSON.parseObject(body);
        JSONArray details = jsonObject.getJSONArray("details");
        List<ProjectBudgetPlanDetail> detailList = new ArrayList<>(details.size());
        for (int i = 0; i < details.size(); i++) {
            ProjectBudgetPlanDetail detail = details.getObject(i, ProjectBudgetPlanDetail.class);
            detailList.add(detail);
        }

        Project project = projectService.getProject(jsonObject.getInteger("projectId"));

        projectBudgetService.saveBudgetPlanDetail(project, detailList);

        return ResponseMsg.buildSuccessMsg("成功");
    }




    /**
     * 待我审核项目列表
     */
    @RequestMapping("/listApprove")
    public String listApprove(@RequestParam(value = "keywords",required = false) String keywords,
                       @RequestParam(value = PAGE_NUMBER, defaultValue = DEFAULT_PAGE_NUMBER) int pageNumber,
                       @RequestParam(value = PAGE_SIZE, defaultValue = DEFAULT_PAGE_SIZE) int pageSize,
                       Map<String, Object> model) {
        //当前登录人的角色类型
        model.put("keywords",keywords);
        model.put("deptList", deptRepository.findAll());
        ConcurrentHashMap<String, String> searchInfo = getSearchInfo(keywords,model);
        model.put("pager",projectService.findMyApproveProjects(searchInfo,InterfaceUtil.getAdminId(),pageNumber,pageSize));
        return "admin/project_list_approve";
    }

    /**
     * 待我审核项目导出
     */
    @RequestMapping("/exportApprove")
    public void exportApprove(@RequestParam(value = "keywords",required = false) String keywords, HttpServletResponse httpServletResponse) throws IOException {
        Map<String, String> searchInfo = getSearchInfo(keywords);
        downloadHeader(httpServletResponse , Utils.generateExcelName("待我审核项目报表"), "application/octet-stream");
        String[] headers = {"项目名称","项目类型","项目状态","审核状态","当前审核人","项目创建者","部门名称","项目开始时间","项目结束时间","最后更新时间"};
        String[] exportColumns = {"name","typeDesc","statusDesc","approveStatusDesc","approveName","creatorName","deptName","startDate","endDate","lastUpdateTime"};
        ExportUtils.exportToExcel(headers, exportColumns, 1, 10000,
                httpServletResponse.getOutputStream(), (pN, pS) -> projectService.findMyApproveProjects(searchInfo, InterfaceUtil.getAdminId(), pN, pS).getList());
    }












    @InitBinder
    public void initBinder(WebDataBinder webDataBinder){
        webDataBinder.addCustomFormatter(new DateFormatter("yyyy-MM-dd"));
    }
}
