package cn.palmte.work.service;

import cn.palmte.work.model.ProcurementType;
import cn.palmte.work.model.ProcurementTypeRepository;
import cn.palmte.work.utils.InterfaceUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import top.jfunc.common.db.QueryHelper;
import top.jfunc.common.db.bean.Page;
import top.jfunc.common.db.utils.Pagination;

import java.util.Date;
import java.util.Map;

@Service
public class ProcurementTypeService {

    @Autowired
    private ProcurementTypeRepository procurementTypeRepository;

    @Autowired
    private Pagination pagination;

    public Page<ProcurementType> list(Map<String, String> searchInfo, int pageNumber, int pageSize){
        QueryHelper queryHelper = new QueryHelper("SELECT *","procurement_type");
        queryHelper.addCondition(searchInfo.containsKey("name"), "name like ?", "%" +
                searchInfo.get("name") + "%");
        queryHelper.addCondition(searchInfo.containsKey("category") && !"-1".equals(searchInfo.get("category")),
                "category=" + searchInfo.get("category"));
        queryHelper.addCondition(searchInfo.containsKey("enabled") && !"-1".equals(searchInfo.get("enabled")),
                "enabled=" + searchInfo.get("enabled"));
        queryHelper.addCondition(searchInfo.containsKey("startTime"), "created_time >= ?", searchInfo.get("startTime") + " 00:00:00");
        queryHelper.addCondition(searchInfo.containsKey("endTime"), "created_time <= ?", searchInfo.get("endTime") + " 23:59:59");
        queryHelper.addOrderProperty("created_time", false);
        return pagination.paginate(queryHelper.getSql(), ProcurementType.class,pageNumber,pageSize);
    }

    public ProcurementType findOne(int id) {
        return procurementTypeRepository.findOne(id);
    }

    public void saveOtUpdate(int procurementTypeId, ProcurementType procurementType) {
        ProcurementType obj = procurementTypeRepository.findOne(procurementTypeId);
        if(null == obj){
            obj = new ProcurementType();
            obj.setName(procurementType.getName());
            obj.setCategory(procurementType.getCategory());
            obj.setEnabled(procurementType.getEnabled());
            obj.setCreatedBy(InterfaceUtil.getAdmin().getRealName());
            obj.setCreatedTime(new Date());
        }else {
            obj.setName(procurementType.getName());
            obj.setCategory(procurementType.getCategory());
            obj.setEnabled(procurementType.getEnabled());
        }
        procurementTypeRepository.saveAndFlush(obj);
    }

    public boolean deleteByIDs(String[] deleteIds) {
        boolean deleted = true;
        try {
            for (String id : deleteIds) {
                procurementTypeRepository.delete(Integer.parseInt(id));
            }
        }catch (Exception e){
            e.printStackTrace();
            deleted = false;
        }

        return deleted;
    }

    public boolean enableOrDisable(int status, int id) {
        ProcurementType one = procurementTypeRepository.findOne(id);
        one.setEnabled(status);
        ProcurementType procurementType = procurementTypeRepository.saveAndFlush(one);

        if(null != procurementType){
            return true;
        }

        return false;
    }
}
